﻿using System.Data;
using System.Reflection;
using Connected.Entities.Annotations;
using Connected.Entities.Storage;
using Connected.Interop;

namespace Connected.Data.Storage
{
    internal static class ReturnValueBinder
	{
		public static void Bind(IStorageWriter w, IStorageOperation operation)
		{
			List<PropertyInfo> properties = null;

			if (w.Operation.Parameters is null)
				return;

			foreach (var parameter in w.Operation.Parameters)
			{
				if (parameter.Direction != ParameterDirection.ReturnValue && parameter.Direction != ParameterDirection.Output)
					continue;

				if (parameter.Value == DBNull.Value)
					continue;

				if (properties is null)
				{
					properties = new List<PropertyInfo>();

					var all = operation.GetType().GetProperties(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);

					foreach (var prop in all)
					{
						if (prop.FindAttribute<ReturnValueAttribute>() is not null)
							properties.Add(prop);
					}
				}

				PropertyInfo property = null;

				if (property is null)
				{
					foreach (var prop in properties)
					{
						if (string.Equals(prop.Name, parameter.Name, StringComparison.Ordinal))
						{
							property = prop;
							break;
						}
					}
				}

				if (property is null)
				{
					foreach (var prop in properties)
					{
						if (string.Equals(prop.Name, parameter.Name, StringComparison.OrdinalIgnoreCase))
						{
							property = prop;
							break;
						}
					}
				}

				if (property is null)
				{
					var candidates = new List<string>
			{
			  parameter.Name.Replace("@", string.Empty)
			};

					foreach (var prop in properties)
					{
						foreach (var candidate in candidates)
						{
							if (string.Equals(prop.Name, candidate, StringComparison.OrdinalIgnoreCase))
							{
								property = prop;
								break;
							}
						}

						if (property is not null)
							break;
					}
				}

				if (property is not null)
				{
					var existingValue = property.GetValue(operation);
					var overwriteAtt = property.FindAttribute<ReturnValueAttribute>();

					switch (overwriteAtt.ValueBehavior)
					{
						case PropertyValueBehavior.OverwriteDefault:
							var defaultValue = property.PropertyType.GetDefault();

							if (Equals(existingValue, defaultValue))
								property.SetValue(operation, parameter.Value);
							break;
						case PropertyValueBehavior.AlwaysOverwrite:
							property.SetValue(operation, parameter.Value);
							break;
					}
				}
			}
		}

	}
}
